#include <iostream>
#include <vector>
#include <ctime>
#include "histogram.hpp"
#include "text.hpp"
#include "svg.hpp"
#include <sstream>
#include "curl.hpp"
#include <string>

using namespace std;

struct Input
{
    vector<double> numbers;
    size_t bin_count{};
};

Input input_data(istream& in, bool prompt)
{
    size_t number_count;
    if (prompt) {cerr << "Enter number count: ";}
    in >> number_count; // Кол-во чисел
    Input inp;
    inp.numbers.resize(number_count);
    for (size_t i = 0; i < number_count; i++)
    {
        if(prompt){cerr << "Enter number №" << i+1 << ": ";}
        in >> inp.numbers[i];
    }
    if(prompt){cerr << "Enter bin count: ";}
    in >> inp.bin_count;
    return inp;
}

size_t
write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    size_t data_size = item_size * item_count;
    (*buffer).write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}

Input
download(const string& adress){
    curl_global_init(CURL_GLOBAL_ALL);
    stringstream buffer;
    CURL* curl = curl_easy_init();
    if(curl)
    {
        CURLcode res;
        curl_easy_setopt(curl, CURLOPT_URL, adress.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        res = curl_easy_perform(curl);
        if (res != CURLE_OK)
        {
            cerr << curl_easy_strerror(res);
            exit(1);
        }
        curl_easy_cleanup(curl);
    }
    curl_global_cleanup();
    return input_data(buffer, false);
}

int main(int argc, char* argv[])
{
    curl_global_init(CURL_GLOBAL_ALL);
    
    Input in;
    if (argc > 1)
        in = download(argv[1]);
    else
        in = input_data(cin, true);
    
    auto bins = make_histogram(in.numbers, in.bin_count);
    show_histogram_svg(bins);
    return 0;
}
